==> crow-2.11.0.tar.bz2: crow-2.11.0/crow/session.h <==
/*
 *  Crow, a GUI designer for GTK+
 *  Copyright (c) 2005 Maxim Udushlivy
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef CROW_SESSION_H
#define CROW_SESSION_H

#include "controller.h"

namespace Crow {

enum SessionOperation { soRemove, soInsert, soUpdate };
enum SessionRole { srNone, srScalar, srVector, srObject };

class Session : public Object {

friend class SessionSupplier;

public:
	static PSession create();

	typedef std::vector<Property *> Properties;
	static SessionRole getRoleForNodes(const Nodes & nodes);
	static SessionRole getRoleForNode(PNode node);
	static Properties findProperties(Controller * controller, const Nodes & nodes);
	static EditorId getEditorId(const Properties & properties);
	static TypeId getPropertyType(const Properties & properties);

	void init(const Nodes & nodes, const IdPath & path, SessionManager * manager);
	void done();

	PEditor getEditor();
	void setEditor(PEditor editor);
	SessionManager * getManager() { return manager; }

	SessionRole getRole() { return role; }
	bool isMultiple();
	std::string getName();

	bool modelEditable();
	bool canEdit();
	bool isModified();
	bool isTouched();
	bool hasToSave();
	bool canDefault();

	bool hasHidden() { return hasFlag(sfHidden); }
	bool hasNoUndo() { return hasFlag(sfNoUndo); }
	bool areWeakLinks() { return haveFlags(sfWeakLink); }
	bool hasLinkOnly() { return hasFlag(sfLinkOnly); }
	bool areTranslatable() { return haveFlags(sfTranslatable); }
	bool isElement();
	bool hasMasterLink();
	bool hasVectorWithMasterLink();
	bool areEmptyLinks();
	TypeId getFinalType();
	TypeId getVectorType();
	Any getScalar();
	Glib::ustring getMeta();
	TypeId getObjectBaseType();
	PNode getLinkTarget();
	TypeId getEntityType();
	EditorId getEditorId();
	bool isFuzzy();

	void resetScalar();
	void resetVector();
	void resetObject();
	void setScalarOnly(Any value);
	void setScalarAndMeta(Any value, const Glib::ustring & meta);
	void setObjectAsLink(PNode object);
	void setObjectAsEntity(TypeId type);

	Property * getProperty1();

	const Nodes * getNodes() { return &nodes; }
	PNode getNode1();

	SessionOperation getOperation() { return operation; }
	const IdPath * getPath() { return &path; }

protected:
	Session();

	bool hasFlag(StateFlags sf);
	bool haveFlags(StateFlags sf);

	Properties getProperties();

private:
	SessionRole role;
	IdPath path;
	Nodes nodes;
	PEditor editor;
	SessionManager * manager;
	SessionOperation operation;

};

class SessionSupplier : public sigc::trackable {

public:
	SessionSupplier();
	virtual ~SessionSupplier() { }

	void setManager(SessionManager * m) { manager= m; }

	void updateSessions();
	void removeSessions();

	void editorsLoad();

	void applySettings();
	void storeSettings();

	PSession findSession(const IdPath & path);
	PSession findSession(PNode node);

	typedef sigc::signal<void, PSession> SignalSessionEvent;
	typedef sigc::signal<void, PSession> SignalAfterLoad;
	typedef sigc::signal<void> SignalPopupMenu;

	SignalSessionEvent & signal_session_event() { return signalSessionEvent; }
	SignalAfterLoad & signal_after_load() { return signalAfterLoad; }
	SignalPopupMenu & signal_popup_menu() { return signalPopupMenu; }

protected:
	SessionManager * getManager() { return manager; }
	const Sessions * getSessions() { return &sessions; }

	virtual void findNodes() { }
 
	void append(PNode cut, PNode node);
	void append(const IdPath & path, PNode node);
	PSession createSession(const Nodes & nodes, const IdPath & path);
	virtual PEditor createEditor(PSession session);
	void stopSession(PSession);

	void sessionEvent(PSession session) { signalSessionEvent.emit(session); }
	void afterLoad(PSession session) { signalAfterLoad.emit(session); }
	void popupMenu() { signalPopupMenu.emit(); }

	typedef std::map<IdPath, Nodes> NodeMap;
	NodeMap * getFound() { return &found; }

private:
	SignalSessionEvent signalSessionEvent;
	SignalAfterLoad signalAfterLoad;
	SignalPopupMenu signalPopupMenu;
	Sessions sessions;
	SessionManager * manager;

	NodeMap found;

};

class HierarchySessionSupplier : public SessionSupplier {

public:
	void findNodes();
	PEditor createEditor(PSession session);

};

class CanvasSessionSupplier : public SessionSupplier {

public:
	void findNodes();
	PEditor createEditor(PSession session);
	PCanvasEditor getEditor();

	bool startPasting(const Glib::ustring & data, bool normal);
	void stopPasting();

	typedef sigc::signal<void> SignalPastingFinished;
	typedef sigc::signal<void, Glib::ustring> SignalStatusChanged;

	SignalPastingFinished & signal_pasting_finished() { return signalPastingFinished; }
	SignalStatusChanged & signal_status_changed() { return signalStatusChanged; }

protected:
	void pastingFinished() { signalPastingFinished.emit(); }
	void statusChanged(const Glib::ustring & text) { signalStatusChanged.emit(text); }

private:
	SignalPastingFinished signalPastingFinished;
	SignalStatusChanged signalStatusChanged;
	PNode current;

};

class SessionSupplierPropertiesBase : public SessionSupplier {

protected:
	void findProperties(PNode cut, PNode node);
	void screenNodes();
	bool mergeable(const Nodes & nodes);
	PEditor createPropertyEditor(PSession session);

};

class PropertiesSessionSupplier : public SessionSupplierPropertiesBase {

public:
	void findNodes();
	PEditor createEditor(PSession session);

};

class SignalsSessionSupplier : public SessionSupplier {

public:
	void findNodes();
	PEditor createEditor(PSession session);

};

class PackingSessionSupplier : public SessionSupplierPropertiesBase {

public:
	void findNodes();
	PEditor createEditor(PSession session);

};

class SessionManager {

public:
	SessionManager();
	~SessionManager();

	void addSupplier(SessionSupplier * supplier);
	void acceptSuppliers();

	Controller * getController() { return &controller; }
	Model * getModel() { return controller.getModel(); }

	const Nodes * getSelection() { return &selection; }
	const Nodes * getReducedSelection() { return &reducedSelection; }
	bool isSelected(PNode node);
	PNode getSingleSelection();
	PNode raiseSelectionParent();
	PNode raiseSelectionMaster();

	void submitSelection(const Nodes &);
	void deleteSelection();

	void begin(UpdateMode um=umNormal);
	void rollback();
	void commit();
	void commit(const Nodes & sel);

	void undo();
	void redo();
	bool canUndo();
	bool canRedo();

	LoadResult load(const Glib::ustring & data);
	void clear();
	void pasteToRoot(const Glib::ustring & data, bool normal);
	Glib::ustring save();
	Glib::ustring copy();

	PSession findSession(PNode node);
	bool isPerforming() { return performing; }
	Container * getContainer(PNode node);
	bool isXYChild(PNode node);
	bool isIndexChild(PNode node);

	typedef Pool<PNode> States;
	States * getStates() { return &states; }

	void applySettings();
	void storeSettings();

	typedef sigc::signal<void> SignalBeforeSessionsUpdate;
	typedef sigc::signal<void> SignalAfterSessionsUpdate;
	typedef sigc::signal<void> SignalSelectionChanged;
	typedef sigc::signal<void> SignalActionStarted;
	typedef sigc::signal<void> SignalActionCompleted;

	SignalBeforeSessionsUpdate & signal_before_sessions_update() { return signalBeforeSessionsUpdate; }
	SignalAfterSessionsUpdate & signal_after_sessions_update() { return signalAfterSessionsUpdate; }
	SignalSelectionChanged & signal_selection_changed() { return signalSelectionChanged; }
	SignalActionStarted & signal_action_started() { return signalActionStarted; }
	SignalActionCompleted & signal_action_completed() { return signalActionCompleted; }

protected:
	void beginAction();
	void endAction();

	void updateSessions();

	bool acceptSelection(const Nodes & nodes);
	void updateSelection(const Nodes & nodes, bool force=false);

private:
	typedef std::list<SessionSupplier *> Suppliers;
	Suppliers suppliers;
	Nodes selection, reducedSelection;
	Controller controller;
	bool performing;
	States states;

	SignalBeforeSessionsUpdate signalBeforeSessionsUpdate;
	SignalAfterSessionsUpdate signalAfterSessionsUpdate;
	SignalSelectionChanged signalSelectionChanged;
	SignalActionStarted signalActionStarted;
	SignalActionCompleted signalActionCompleted;

};

}

#endif

==> crow-2.11.3.tar.bz2: crow-2.11.3/crow/session.h <==
/*
 *  Crow, a GUI designer for GTK+
 *  Copyright (c) 2005-2008 Maxim Udushlivy
 *
 *  Permission is hereby granted, free of charge, to any person 
 *  obtaining a copy of this software and associated documentation 
 *  files (the "Software"), to deal in the Software without 
 *  restriction, including without limitation the rights to use, copy, 
 *  modify, merge, publish, distribute, sublicense, and/or sell copies 
 *  of the Software, and to permit persons to whom the Software is 
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be 
 *  included in all copies or substantial portions of the Software.
 *
 *  The software is provided "as is", without warranty of any kind, 
 *  express or implied, including but not limited to the warranties of 
 *  merchantability, fitness for a particular purpose and 
 *  noninfringement. In no event shall the authors or copyright 
 *  holders be liable for any claim, damages or other liability, 
 *  whether in an action of contract, tort or otherwise, arising from, 
 *  out of or in connection with the software or the use or other 
 *  dealings in the software.
 */

#ifndef CROW_SESSION_H
#define CROW_SESSION_H

#include "controller.h"

namespace Crow {

enum SessionOperation { soRemove, soInsert, soUpdate };
enum SessionRole { srNone, srScalar, srVector, srObject };

class Session : public Object {

friend class SessionSupplier;

public:
	static PSession create();

	typedef std::vector<Property *> Properties;
	static SessionRole getRoleForNodes(const Nodes & nodes);
	static SessionRole getRoleForNode(PNode node);
	static Properties findProperties(Controller * controller, const Nodes & nodes);
	static EditorId getEditorId(const Properties & properties);
	static TypeId getPropertyType(const Properties & properties);

	void init(const Nodes & nodes, const IdPath & path, SessionManager * manager);
	void done();

	PEditor getEditor();
	void setEditor(PEditor editor);
	SessionManager * getManager() { return manager; }

	SessionRole getRole() { return role; }
	bool isMultiple();
	std::string getName();

	bool modelEditable();
	bool canEdit();
	bool isModified();
	bool isTouched();
	bool hasToSave();
	bool canDefault();

	bool hasHidden() { return hasFlags(sfHidden); }
	bool hasNoUndo() { return hasFlags(sfNoUndo); }
	bool areWeakLinks() { return haveFlags(sfWeakLink); }
	bool hasLinkOnly() { return hasFlags(sfLinkOnly); }
	bool areTranslatable() { return haveFlags(sfTranslatable); }
	bool allowTopMasters() { return haveFlags(sfAllowTM); }
	bool isElement();
	bool hasMasterLink();
	bool hasVectorWithMasterLink();
	bool areEmptyLinks();
	TypeId getFinalType();
	TypeId getVectorType();
	Any getScalar();
	Glib::ustring getMeta();
	TypeId getObjectBaseType();
	PNode getLinkTarget();
	TypeId getEntityType();
	EditorId getEditorId();
	bool isFuzzy();

	void resetScalar();
	void resetVector();
	void resetObject();
	void setScalarOnly(Any value);
	void setScalarAndMeta(Any value, const Glib::ustring & meta);
	void setObjectAsLink(PNode object);
	void setObjectAsEntity(TypeId type);

	Property * getProperty1();

	const Nodes * getNodes() { return &nodes; }
	PNode getNode1();

	SessionOperation getOperation() { return operation; }
	const IdPath * getPath() { return &path; }

protected:
	Session();

	bool hasFlags(StateFlags sf);
	bool haveFlags(StateFlags sf);

	Properties getProperties();

private:
	SessionRole role;
	IdPath path;
	Nodes nodes;
	PEditor editor;
	SessionManager * manager;
	SessionOperation operation;

};

class SessionSupplier : public sigc::trackable {

public:
	SessionSupplier();
	virtual ~SessionSupplier() { }

	void setManager(SessionManager * m) { manager= m; }

	void updateSessions();
	void removeSessions();

	void editorsLoad();

	void applySettings();
	void storeSettings();

	PSession findSession(const IdPath & path);
	PSession findSession(PNode node);

	typedef sigc::signal<void, PSession> SignalSessionEvent;
	typedef sigc::signal<void, PSession> SignalAfterLoad;
	typedef sigc::signal<void> SignalPopupMenu;

	SignalSessionEvent & signal_session_event() { return signalSessionEvent; }
	SignalAfterLoad & signal_after_load() { return signalAfterLoad; }
	SignalPopupMenu & signal_popup_menu() { return signalPopupMenu; }

protected:
	SessionManager * getManager() { return manager; }
	const Sessions * getSessions() { return &sessions; }

	virtual void findNodes() { }
 
	void append(PNode cut, PNode node);
	void append(const IdPath & path, PNode node);
	PSession createSession(const Nodes & nodes, const IdPath & path);
	virtual PEditor createEditor(PSession session);
	void stopSession(PSession);

	void sessionEvent(PSession session) { signalSessionEvent.emit(session); }
	void afterLoad(PSession session) { signalAfterLoad.emit(session); }
	void popupMenu() { signalPopupMenu.emit(); }

	typedef std::map<IdPath, Nodes> NodeMap;
	NodeMap * getFound() { return &found; }

private:
	SignalSessionEvent signalSessionEvent;
	SignalAfterLoad signalAfterLoad;
	SignalPopupMenu signalPopupMenu;
	Sessions sessions;
	SessionManager * manager;

	NodeMap found;

};

class HierarchySessionSupplier : public SessionSupplier {

public:
	void findNodes();
	PEditor createEditor(PSession session);

};

class CanvasSessionSupplier : public SessionSupplier {

public:
	void findNodes();
	PEditor createEditor(PSession session);
	PCanvasEditor getEditor();

	bool startPasting(const Glib::ustring & data, bool normal);
	void stopPasting();

	typedef sigc::signal<void> SignalPastingFinished;
	typedef sigc::signal<void, Glib::ustring> SignalStatusChanged;

	SignalPastingFinished & signal_pasting_finished() { return signalPastingFinished; }
	SignalStatusChanged & signal_status_changed() { return signalStatusChanged; }

protected:
	void pastingFinished() { signalPastingFinished.emit(); }
	void statusChanged(const Glib::ustring & text) { signalStatusChanged.emit(text); }

private:
	SignalPastingFinished signalPastingFinished;
	SignalStatusChanged signalStatusChanged;
	PNode current;

};

class SessionSupplierPropertiesBase : public SessionSupplier {

protected:
	void findProperties(PNode cut, PNode node);
	void screenNodes();
	bool mergeable(const Nodes & nodes);
	PEditor createPropertyEditor(PSession session);

};

class PropertiesSessionSupplier : public SessionSupplierPropertiesBase {

public:
	void findNodes();
	PEditor createEditor(PSession session);

};

class SignalsSessionSupplier : public SessionSupplier {

public:
	void findNodes();
	PEditor createEditor(PSession session);

};

class PackingSessionSupplier : public SessionSupplierPropertiesBase {

public:
	void findNodes();
	PEditor createEditor(PSession session);

};

class SessionManager {

public:
	SessionManager();
	~SessionManager();

	void addSupplier(SessionSupplier * supplier);
	void acceptSuppliers();

	Controller * getController() { return &controller; }
	Model * getModel() { return controller.getModel(); }

	const Nodes * getSelection() { return &selection; }
	const Nodes * getReducedSelection() { return &reducedSelection; }
	bool isSelected(PNode node);
	PNode getSingleSelection();
	PNode raiseSelectionParent();
	PNode raiseSelectionMaster();

	void submitSelection(const Nodes &);
	void deleteSelection();

	void begin(UpdateMode um=umNormal);
	void rollback();
	void commit();
	void commit(const Nodes & sel);

	void undo();
	void redo();
	bool canUndo();
	bool canRedo();

	LoadResult load(const Glib::ustring & data);
	void clear();
	void pasteToRoot(const Glib::ustring & data, bool normal);
	Glib::ustring save();
	Glib::ustring copy();

	PSession findSession(PNode node);
	bool isPerforming() { return performing; }
	Container * getContainer(PNode node);
	bool isXYChild(PNode node);
	bool isIndexChild(PNode node);

	typedef Pool<PNode> States;
	States * getStates() { return &states; }

	void applySettings();
	void storeSettings();

	typedef sigc::signal<void> SignalBeforeSessionsUpdate;
	typedef sigc::signal<void> SignalAfterSessionsUpdate;
	typedef sigc::signal<void> SignalSelectionChanged;
	typedef sigc::signal<void> SignalActionStarted;
	typedef sigc::signal<void> SignalActionCompleted;

	SignalBeforeSessionsUpdate & signal_before_sessions_update() { return signalBeforeSessionsUpdate; }
	SignalAfterSessionsUpdate & signal_after_sessions_update() { return signalAfterSessionsUpdate; }
	SignalSelectionChanged & signal_selection_changed() { return signalSelectionChanged; }
	SignalActionStarted & signal_action_started() { return signalActionStarted; }
	SignalActionCompleted & signal_action_completed() { return signalActionCompleted; }

protected:
	void beginAction();
	void endAction();

	void updateSessions();

	bool acceptSelection(const Nodes & nodes);
	void updateSelection(const Nodes & nodes, bool force=false);

private:
	typedef std::list<SessionSupplier *> Suppliers;
	Suppliers suppliers;
	Nodes selection, reducedSelection;
	Controller controller;
	bool performing;
	States states;

	SignalBeforeSessionsUpdate signalBeforeSessionsUpdate;
	SignalAfterSessionsUpdate signalAfterSessionsUpdate;
	SignalSelectionChanged signalSelectionChanged;
	SignalActionStarted signalActionStarted;
	SignalActionCompleted signalActionCompleted;

};

}

#endif
